// Copyright 2006, Massachusetts Institute of Technology.
// The use of this code is permitted for research only. There is
// absolutely no warranty for this software.
//
// Author: John Lee (jjl@mit.edu)
//

#include <assert.h>
#include <vector>
#include "experiment/experiment.h"
#include "util/labeled-index.h"
#include "kernel/kernel-matrix.h"

using namespace libpmk;
using namespace libpmk_util;

Experiment::Experiment(vector<LabeledIndex> training,
                       vector<LabeledIndex> testing,
                       const KernelMatrix& kernel) :
   training_(training), testing_(testing), kernel_(kernel) {
   predictions_.resize(testing_.size());
}

int Experiment::GetPrediction(int test_index) const {
   assert(test_index < (int)testing_.size());
   return predictions_[test_index];
}

void Experiment::SetPrediction(int test_index, int prediction) {
   assert(test_index < (int)testing_.size());
   predictions_[test_index] = prediction;
}

int Experiment::GetNumCorrect() const {
   int count = 0;
   for (int ii = 0; ii < (int)testing_.size(); ++ii) {
      if (testing_[ii].label == predictions_[ii]) {
         ++count;
      }
   }
   return count;
}

int Experiment::GetNumCorrect(int label) const {
   int count = 0;
   for (int ii = 0; ii < (int)testing_.size(); ++ii) {
      if (testing_[ii].label == label &&
          testing_[ii].label == predictions_[ii]) {
         ++count;
      }
   }
   return count;
}

int Experiment::GetNumTestExamples() const {
   return testing_.size();
}

int Experiment::GetNumTestExamples(int label) const {
   int count = 0;
   for (int ii = 0; ii < (int)testing_.size(); ++ii) {
      if (testing_[ii].label == label) {
         ++count;
      }
   }
   return count;
}

double Experiment::GetAccuracy() const {
   return (static_cast<double>(GetNumCorrect()) /
           static_cast<double>(GetNumTestExamples()));
}

double Experiment::GetAccuracy(int label) const {
   return (static_cast<double>(GetNumCorrect(label)) /
           static_cast<double>(GetNumTestExamples(label)));
}

double Experiment::GetKernelValue(int row, int col) const {
   return kernel_.at(row, col);
}

double Experiment::GetKernelValue(const LabeledIndex& row,
                                  const LabeledIndex& col) const {
   return kernel_.at(row.index, col.index);
}
