// Copyright 2007, Massachusetts Institute of Technology.
// The use of this code is permitted for research only. There is
// absolutely no warranty for this software.
//
// Author: John Lee (jjl@mit.edu)
//

#include "histograms/bin.h"
#include "util/sparse-vector.h"

namespace libpmk {

Bin::Bin(const LargeIndex& index) : index_(index), size_(0), count_(0),
                                    parent_(NULL), next_sibling_(NULL),
                                    first_child_(NULL) { }

const LargeIndex& Bin::GetIndex() const {
   return index_;
}

double Bin::GetSize() const {
   return size_;
}

double Bin::SetSize(double size) {
   double old_size = size_;
   size_ = size;
   return old_size;
}

double Bin::IncrementCount() {
   return count_ += 1;
}

double Bin::DecrementCount() {
   return count_ -= 1;
}

double Bin::SetCount(double count) {
   return (count_ = count);
}

double Bin::GetCount() const {
   return count_;
}

void Bin::SetParent(Bin* parent) {
   parent_ = parent;
}

void Bin::SetNextSibling(Bin* next_sibling) {
   next_sibling_ = next_sibling;
}

void Bin::SetFirstChild(Bin* first_child) {
   first_child_ = first_child;
}

Bin* Bin::GetParent() {
   return parent_;
}

Bin* Bin::GetNextSibling() {
   return next_sibling_;
}

Bin* Bin::GetFirstChild() {
   return first_child_;
}

bool Bin::HasParent() const {
   return parent_ != NULL;
}

bool Bin::HasNextSibling() const {
   return next_sibling_ != NULL;
}

bool Bin::HasChild() const {
   return first_child_ != NULL;
}

bool Bin::operator<(const Bin& other) const {
   return index_ < other.index_;
}

bool Bin::CompareBins(const Bin* a, const Bin* b) {
   return a->GetIndex() < b->GetIndex();
}
}  // namespace libpmk
