// Copyright 2007, Massachusetts Institute of Technology.
// The use of this code is permitted for research only. There is
// absolutely no warranty for this software.
//
// Author: John Lee (jjl@mit.edu)
//
// Takes a point set list and the output of HierarchicalClusterer, to create
// a vocab-guided pyramid.

#include <iostream>
#include <fstream>
#include "pyramids/input-specific-vg-pyramid-maker.h"
#include "pyramids/global-vg-pyramid-maker.h"
#include "clustering/hierarchical-clusterer.h"
#include "histograms/multi-resolution-histogram.h"
#include "util/distance-computer.h"
#include "util/bin-weight-scheme.h"
#include "point_set/mutable-point-set-list.h"

using namespace std;
using namespace libpmk;

void usage(const char* exec_name) {
   cerr << "Usage: " << exec_name << " training.psl data.psl "
        << "clusters.hc output.mrh\n\n";
   cerr << "<training.psl>: A PointSetList file, where each PointSet contains\n"
        << "              features for one image\n";
   cerr << "<data.psl>: A PointSetList file, where each PointSet contains\n"
        << "              features for one image\n";
   cerr << "<clusters.hc>:  The result of running HierarchicalClusterer\n";
   cerr << "<output.mrh>: Where to write the pyramids to\n";
}

int main(int argc, char** argv) {
   if (argc != 5) {
      usage(argv[0]);
      exit(0);
   }

   string clustered_file(argv[1]);
   string point_set_file(argv[2]);
   string clusters_file(argv[3]);
   string output_file(argv[4]);

   MutablePointSetList clustered;
   ifstream input_clustered_stream(clustered_file.c_str(), ios::binary);
   clustered.ReadFromStream(input_clustered_stream);
   input_clustered_stream.close();

   MutablePointSetList psl;
   ifstream input_psl_stream(point_set_file.c_str(), ios::binary);
   psl.ReadFromStream(input_psl_stream);
   input_psl_stream.close();

   HierarchicalClusterer clusterer;
   ifstream input_stream(clusters_file.c_str(), ios::in | ios::binary);
   clusterer.ReadFromStream(input_stream);
   input_stream.close();

   L2DistanceComputer dist;
   GlobalVGPyramidMaker vgpm(clusterer, dist);
   
   cout << "Preprocessing" << endl;
   vgpm.Preprocess(clustered);

   cout << "Making pyramids" << endl;
   vector<MultiResolutionHistogram*> vec = vgpm.MakePyramids(psl);

   ofstream output_stream(output_file.c_str(), ios::binary | ios::trunc);
   MultiResolutionHistogram::WriteToStream(output_stream, vec);
   output_stream.close();
   return 0;
}
