// Copyright 2007, Massachusetts Institute of Technology.
// The use of this code is permitted for research only. There is
// absolutely no warranty for this software.
//
// Author: John Lee (jjl@mit.edu)
//
// Usage: random-sampling-merger <X> <output_labels> <output_ptsets> <N>
//             <psl1> <psl2> ... <pslN>
// Takes N input point-set-list files, randomly samples X (0 < X <= 1)
// points in each input point set. It will then combine all the point
// sets into one file and generate a label file.

#include <assert.h>
#include <math.h>
#include <iostream>
#include <fstream>
#include "point_set/mutable-point-set-list.h"

using namespace std;
using namespace libpmk;

void usage(const char* argv) {
   fprintf(stderr, "Usage: %s <output_labels> <output_ptsets> <N> <psl1> ... <pslN>\n", argv);
}


int main(int argc, char** argv) {
   if (argc < 4) {
      usage(argv[0]);
      exit(0);
   }

   string label_file(argv[1]);
   string output_file(argv[2]);
   int num_inputs = atoi(argv[3]);
   
   if (argc != 4 + num_inputs) {
      usage(argv[0]);
      exit(0);
   }


   MutablePointSetList all_point_sets;
   vector<int> labels;

   for (int ii = 4; ii < argc; ++ii) {
      int current_label = ii - 3;
      printf("Doing set %d (%s)\n", current_label, argv[ii]);
      MutablePointSetList point_sets;
      ifstream input_stream(argv[ii], ios::binary);
      point_sets.ReadFromStream(input_stream);
      input_stream.close();
      printf("Finished reading point sets.\n");      
      
      for (int ii = 0; ii < point_sets.GetNumPointSets(); ++ii) {
         all_point_sets.AddPointSet(point_sets[ii]);
         labels.push_back(current_label);
      }
   }



   assert(all_point_sets.GetNumPointSets() == (int)labels.size());
   
   printf("Writing a total of %d points.\n", all_point_sets.GetNumPoints());
   ofstream output_stream(output_file.c_str(), ios::binary | ios::trunc);
   all_point_sets.WriteToStream(output_stream);

   FILE* f = fopen(label_file.c_str(), "w");
   fprintf(f, "%d\n", (int)labels.size());
   for (int ii = 0; ii < (int)labels.size(); ++ii) {
      fprintf(f, "%d\n", labels[ii]);
   }
   fclose(f);
   
   return 0;

}
