// Copyright 2007, Massachusetts Institute of Technology.
// The use of this code is permitted for research only. There is
// absolutely no warranty for this software.
//
// Author: John Lee (jjl@mit.edu)
//
// This program takes point sets and creates uniformly-sized pyramids.

#include <math.h>
#include <assert.h>
#include <stdlib.h>
#include <time.h>
#include <iostream>
#include <fstream>
#include "histograms/bin.h"
#include "pyramids/uniform-pyramid-maker.h"
#include "point_set/mutable-point-set-list.h"

using namespace libpmk;

void usage(const char* exec_name) {
   cerr << "Usage: " << exec_name << " input.psl output.mrh f s d\n\n";
   cerr << "<input.psl>: A PointSetList file, where each PointSet contains\n"
        << "             features for one image\n";
   cerr << "<output.mrh>: The resulting pyramids\n";
   cerr << "<f>:        (double) The finest side length\n";
   cerr << "<s>:        (double) Side length factor\n";
   cerr << "<d>:        (int) Scale points by multiplying by 10^(d)\n";
}


int main(int argc, char** argv) {
   if (argc != 6) {
      usage(argv[0]);
      exit(0);
   }

   string input_file(argv[1]);
   string output_file(argv[2]);
   double finest_side_length = atof(argv[3]);
   double side_length_factor = atof(argv[4]);
   int discretize_order = atoi(argv[5]);

   // Set the random seed for translations
   srand(time(NULL));

   MutablePointSetList psl;
   ifstream input_stream(input_file.c_str(), ios::binary);
   psl.ReadFromStream(input_stream);
   input_stream.close();
   UniformPyramidMaker upm;

   upm.Preprocess(psl, finest_side_length, side_length_factor,
                  discretize_order, true, true);

   printf("Making pyramids: num levels is %d\n", upm.GetNumLevels());
   vector<MultiResolutionHistogram*> vec = upm.MakePyramids(psl);
   
   ofstream output_stream(output_file.c_str(), ios::binary | ios::trunc);
   MultiResolutionHistogram::WriteToStream(output_stream, vec);
   output_stream.close();
   return 0;
}
