// Copyright 2007, Massachusetts Institute of Technology.
// The use of this code is permitted for research only. There is
// absolutely no warranty for this software.
//
// Author: John Lee (jjl@mit.edu)
//
// Given a file with Pyramids, computes the pyramid match kernel.

#include <stdio.h>
#include <math.h>
#include <iostream>
#include <fstream>
#include "histograms/multi-resolution-histogram.h"
#include "pyramids/pyramid-matcher.h"
#include "kernel/kernel-matrix.h"
#include "util/bin-weight-scheme.h"

using namespace std;
using namespace libpmk;
using namespace libpmk_util;

void usage(const char* exec_name) {
   cerr << "Usage: " << exec_name << " input.mrh output.kern\n\n";
   cerr << "<input.mrh>:  Input pyramids\n";
   cerr << "<output.kern>: Where to write the resulting kernel to\n";
}

int main(int argc, char** argv) {
   if (argc != 3) {
      usage(argv[0]);
      exit(0);
   }

   string input_file(argv[1]);
   string output_file(argv[2]);

   ifstream input_stream(input_file.c_str(), ios::binary);
   vector<MultiResolutionHistogram*> vec(
      MultiResolutionHistogram::ReadFromStream(input_stream));
   input_stream.close();

   KernelMatrix km(vec.size());
   
   printf("Finished reading.\n");
   for (int ii = 0; ii < (int)vec.size(); ++ii) {
      clock_t time_start = clock();
      for (int jj = 0; jj <= ii; ++jj) {
         km.at(ii, jj) = PyramidMatcher::GetPyramidMatchSimilarity(*vec[ii], *vec[jj], BIN_WEIGHT_GLOBAL);
      }
      clock_t time_end = clock();
      double row_time = (double)(time_end - time_start) / CLOCKS_PER_SEC;
      double mean_time = row_time / (ii+1);
      fprintf(stderr, "Row %d took\t%lf seconds:\t%lf sec per example\n", ii, row_time, mean_time);


   }

   km.Normalize();
   
   ofstream output_stream(output_file.c_str(), ios::binary | ios::trunc);
   km.WriteToStream(output_stream);
   output_stream.close();
   return 0;
}
